#!/bin/bash

# search qemu executable and NIC rom in the Qemu package
#export PATH="/opt/qemu-1.2.0/bin:$PATH"
#QEMU=$(which qemu-system-i386)
#ROM=$(rpm -ql qemu | grep pxe-rtl8139.rom)

QEMU=$HOME/qemu/i386-softmmu/qemu-system-i386
ROM=$HOME/qemu/pc-bios/pxe-rtl8139.rom

set -e

obj="$(pwd -P)/out"
test -d "$obj"
test -x "$QEMU"
test -r "$ROM"

# check we have the bios file in out directory
test -r rtl8139.rom

# run qemu with our bios
biosdir=$obj/biosdir
rm -rf $biosdir
cp -rsL "$(dirname $ROM)" $biosdir
rm -f "$biosdir/pxe-rtl8139.rom"
cp rtl8139.rom "$biosdir/pxe-rtl8139.rom"

# new version of Qemu use efi-rtl8139.rom name
rm -f "$biosdir/efi-rtl8139.rom"
cp rtl8139.rom "$biosdir/efi-rtl8139.rom"

# print an error and exit
error() {
	echo "$*" >&2
	exit 1
}

# create a NBP file to print a given string
# arg1: filename
# other args: string to print
print_set() {
	local out
	out="$1"
	shift
	{
		cat $obj/print.0
		echo -n "$*"
		echo -ne '\r\n\0'
	} > "$obj/tftp/$out"
}

# launch qemu but wait maximun for 12 seconds
# This qemu will write screen output to the serial and then serial to a file
# Various configuration changes could be done like timeout, network and
# bootfile
launch_qemu() {
	local timeout bootfile pid netarg order cdrom
	timeout=12
	bootfile=/print.0
	order=nd
	cdrom="$obj/shutdown.iso"
	OPTIND=1
	while getopts "t:b:B:" name "$@"; do
		case $name in
		t)	timeout="$OPTARG";;
		b)	bootfile="$OPTARG";;
		B)	order=d
			cdrom="$OPTARG"
			;;
		?)	exit 2;;
		esac
	done

	netarg="user,tftp=$obj/tftp,bootfile=$bootfile"
	rm -f $out

	# launch a command and wait
	exec 9> $obj/timeout.$$
	flock -n 9
	$QEMU -L $biosdir -nographic -device sga -monitor null -serial file:$out -cdrom "$cdrom" -boot order=$order -net nic,model=rtl8139 -net "$netarg" &
	pid=$!
	exec 9>&-

	flock -w $timeout $obj/timeout.$$ -c 'true' || true
	rm -f $obj/timeout.$$
	kill $pid &> /dev/null || true
	test -r $out
}

test_start() {
	echo "TEST $1"
	echo "----------"
	shift
	echo "$*"
}

# setup our tftp directory
rm -rf $obj/tftp
mkdir $obj/tftp

out=$obj/out.txt
rm -f $out

# TEST 1
test_start 1 "see if it boot with gPXE and execute our program"
print_set print.0 "This is printed from print.0"
launch_qemu
grep -q '[gi]PXE' $out || error "gPXE not found on output, error rewriting output"
echo gPXE found!
grep -q "This is printed from print.0" $out || error "gPXE does not start our NBP"


# TEST 2
# Name must be correctly quoted inside iPXE
# this test test name from DHCP containing #
test_start 2 "test if name contains a # from DHCP"
print_set "test#123" "Second test DHCP"
launch_qemu -b "/test#123"
grep -q "Second test DHCP" $out || error "Unable to load files with #"


# TEST 3
# Name must be correctly quoted inside iPXE
# this test test name from PXE call containing #
# recent changes use different paths in code to do quoting
test_start 3 "test if name contains a # from PXE call"
rm -f $obj/tftp/*
cp $obj/testrestart.2 $obj/tftp/
print_set "test#456" "Second test PXE"
launch_qemu -b "testrestart.2"
grep -q "Second test PXE" $out || error "Unable to load files with #"


# TEST 4
test_start 4 "Test com file not loaded as com"
rm -f $obj/tftp/*.0
cp $obj/test.0 $obj/tftp/test.com
launch_qemu -b "/test.com"
grep -q "CS:IP " $out || error "program not executed"
if grep -q "CS:IP .*:0100" $out; then
	error "file names as *.com detected as com file"
fi


# TEST 5
# This test load an initial NBP that load another NBP which
# load another one (that print a message and halt the machine)
# Test that the first and second use same stack
test_start 5 "Test restarting the PXE NBP"
cp $obj/testrestart.{0,1} $obj/tftp/
print_set "abortpxe.com" "Boot aborted"
launch_qemu -b "testrestart.0"
grep -q "Boot aborted" $out || error "boot has not been completed"
test "$(grep "SS:SP" $out | wc -l)" = "2" || error "not booted the two initial NBPs"
test "$(grep "SS:SP" $out | head -1)" = "$(grep "SS:SP" $out | tail -1)" || error "stack has been changed"

